package cn.com.infosec.crypto.tls;

import cn.com.infosec.asn1.x509.RSAPublicKeyStructure;
import cn.com.infosec.crypto.InvalidCipherTextException;
import cn.com.infosec.crypto.encodings.PKCS1Encoding;
import cn.com.infosec.crypto.engines.RSABlindedEngine;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import cn.com.infosec.crypto.params.RSAKeyParameters;
import cn.com.infosec.crypto.prng.ThreadedSeedGenerator;
import cn.com.infosec.util.BigIntegers;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigInteger;
import java.security.SecureRandom;

public class TlsProtocolHandler {
   private static final short RL_CHANGE_CIPHER_SPEC = 20;
   private static final short RL_ALERT = 21;
   private static final short RL_HANDSHAKE = 22;
   private static final short RL_APPLICATION_DATA = 23;
   private static final short HP_HELLO_REQUEST = 0;
   private static final short HP_CLIENT_HELLO = 1;
   private static final short HP_SERVER_HELLO = 2;
   private static final short HP_CERTIFICATE = 11;
   private static final short HP_SERVER_KEY_EXCHANGE = 12;
   private static final short HP_CERTIFICATE_REQUEST = 13;
   private static final short HP_SERVER_HELLO_DONE = 14;
   private static final short HP_CERTIFICATE_VERIFY = 15;
   private static final short HP_CLIENT_KEY_EXCHANGE = 16;
   private static final short HP_FINISHED = 20;
   private static final short CS_CLIENT_HELLO_SEND = 1;
   private static final short CS_SERVER_HELLO_RECEIVED = 2;
   private static final short CS_SERVER_CERTIFICATE_RECEIVED = 3;
   private static final short CS_SERVER_KEY_EXCHANGE_RECEIVED = 4;
   private static final short CS_CERTIFICATE_REQUEST_RECEIVED = 5;
   private static final short CS_SERVER_HELLO_DONE_RECEIVED = 6;
   private static final short CS_CLIENT_KEY_EXCHANGE_SEND = 7;
   private static final short CS_CLIENT_CHANGE_CIPHER_SPEC_SEND = 8;
   private static final short CS_CLIENT_FINISHED_SEND = 9;
   private static final short CS_SERVER_CHANGE_CIPHER_SPEC_RECEIVED = 10;
   private static final short CS_DONE = 11;
   protected static final short AP_close_notify = 0;
   protected static final short AP_unexpected_message = 10;
   protected static final short AP_bad_record_mac = 20;
   protected static final short AP_decryption_failed = 21;
   protected static final short AP_record_overflow = 22;
   protected static final short AP_decompression_failure = 30;
   protected static final short AP_handshake_failure = 40;
   protected static final short AP_bad_certificate = 42;
   protected static final short AP_unsupported_certificate = 43;
   protected static final short AP_certificate_revoked = 44;
   protected static final short AP_certificate_expired = 45;
   protected static final short AP_certificate_unknown = 46;
   protected static final short AP_illegal_parameter = 47;
   protected static final short AP_unknown_ca = 48;
   protected static final short AP_access_denied = 49;
   protected static final short AP_decode_error = 50;
   protected static final short AP_decrypt_error = 51;
   protected static final short AP_export_restriction = 60;
   protected static final short AP_protocol_version = 70;
   protected static final short AP_insufficient_security = 71;
   protected static final short AP_internal_error = 80;
   protected static final short AP_user_canceled = 90;
   protected static final short AP_no_renegotiation = 100;
   protected static final short AL_warning = 1;
   protected static final short AL_fatal = 2;
   private static final byte[] emptybuf = new byte[0];
   private static final String TLS_ERROR_MESSAGE = "Internal TLS error, this could be an attack";
   private ByteQueue applicationDataQueue;
   private ByteQueue changeCipherSpecQueue;
   private ByteQueue alertQueue;
   private ByteQueue handshakeQueue;
   private RecordStream rs;
   private SecureRandom random;
   private RSAKeyParameters serverRsaKey;
   private TlsInputStream tlsInputStream;
   private TlsOuputStream tlsOutputStream;
   private boolean closed;
   private boolean failedWithError;
   private boolean appDataReady;
   private byte[] clientRandom;
   private byte[] serverRandom;
   private byte[] ms;
   private TlsCipherSuite choosenCipherSuite;
   private BigInteger Yc;
   private byte[] pms;
   private CertificateVerifyer verifyer;
   private short connection_state;

   public TlsProtocolHandler(InputStream is, OutputStream os) {
      this.applicationDataQueue = new ByteQueue();
      this.changeCipherSpecQueue = new ByteQueue();
      this.alertQueue = new ByteQueue();
      this.handshakeQueue = new ByteQueue();
      this.serverRsaKey = null;
      this.tlsInputStream = null;
      this.tlsOutputStream = null;
      this.closed = false;
      this.failedWithError = false;
      this.appDataReady = false;
      this.choosenCipherSuite = null;
      this.verifyer = null;
      ThreadedSeedGenerator var3;
      var3 = new ThreadedSeedGenerator.<init>();
      SecureRandom var4;
      SecureRandom var10001 = var4 = new SecureRandom;
      var4.<init>();
      this.random = var4;
      var10001.setSeed(var3.generateSeed(20, true));
      this.rs = new RecordStream(this, is, os);
   }

   public TlsProtocolHandler(InputStream is, OutputStream os, SecureRandom sr) {
      ByteQueue var4;
      var4 = new ByteQueue.<init>();
      this.applicationDataQueue = var4;
      var4 = new ByteQueue.<init>();
      this.changeCipherSpecQueue = var4;
      var4 = new ByteQueue.<init>();
      this.alertQueue = var4;
      var4 = new ByteQueue.<init>();
      this.handshakeQueue = var4;
      this.serverRsaKey = null;
      this.tlsInputStream = null;
      this.tlsOutputStream = null;
      this.closed = false;
      this.failedWithError = false;
      this.appDataReady = false;
      this.choosenCipherSuite = null;
      this.verifyer = null;
      this.random = sr;
      RecordStream sr1;
      sr1 = new RecordStream.<init>(this, is, os);
      this.rs = sr1;
   }

   private void processHandshake() {
      boolean var1;
      do {
         var1 = false;
         if (this.handshakeQueue.size() >= 4) {
            byte[] var2 = new byte[4];
            this.handshakeQueue.read(var2, 0, 4, 0);
            ByteArrayInputStream var10001 = new ByteArrayInputStream(var2);
            int var3 = TlsUtils.readUint8(var10001);
            int var4 = TlsUtils.readUint24(var10001);
            int var5;
            if (this.handshakeQueue.size() >= (var5 = var4 + 4)) {
               byte[] var6 = new byte[var4];
               this.handshakeQueue.read(var6, 0, var4, 4);
               this.handshakeQueue.removeData(var5);
               if (var3 != 20) {
                  this.rs.hash1.update(var2, 0, 4);
                  this.rs.hash2.update(var2, 0, 4);
                  this.rs.hash1.update(var6, 0, var4);
                  this.rs.hash2.update(var6, 0, var4);
               }

               ByteArrayInputStream var16;
               var16 = new ByteArrayInputStream.<init>(var6);
               byte[] var20;
               if (var3 != 2) {
                  int var15;
                  byte[] var25;
                  byte[] var26;
                  if (var3 != 20) {
                     RSAKeyParameters var19;
                     byte[] var10006;
                     byte[] var23;
                     byte[] var10008;
                     BigInteger var27;
                     BigInteger var36;
                     byte[] var39;
                     PKCS1Encoding var40;
                     byte[] var44;
                     switch (var3) {
                        case 11:
                           if (this.connection_state != 2) {
                              this.failWithError((short)2, (short)10);
                           } else {
                              Certificate var37 = Certificate.parse(var16);
                              this.assertEmpty(var16);
                              if (!this.verifyer.isValid(var37.getCerts())) {
                                 this.failWithError((short)2, (short)90);
                              }

                              Certificate var45 = var37;
                              RSAPublicKeyStructure var38 = null;

                              label118: {
                                 RSAPublicKeyStructure var48;
                                 try {
                                    var48 = RSAPublicKeyStructure.getInstance(var45.certs[0].getTBSCertificate().getSubjectPublicKeyInfo().getPublicKey());
                                 } catch (Exception var11) {
                                    this.failWithError((short)2, (short)43);
                                    break label118;
                                 }

                                 var38 = var48;
                              }

                              RSAKeyParameters var52 = var19 = new RSAKeyParameters;
                              var36 = var38.getModulus();
                              var27 = var38.getPublicExponent();
                              var52.<init>(false, var36, var27);
                              this.serverRsaKey = var19;
                              this.connection_state = 3;
                              var1 = true;
                           }
                           break;
                        case 12:
                           if (this.connection_state != 3) {
                              this.failWithError((short)2, (short)10);
                           } else {
                              if (this.choosenCipherSuite.getKeyExchangeAlgorithm() != 5) {
                                 this.failWithError((short)2, (short)10);
                              }

                              TlsUtils.readFully(var23 = new byte[var15 = TlsUtils.readUint16(var16)], var16);
                              TlsUtils.readFully(var26 = new byte[var4 = TlsUtils.readUint16(var16)], var16);
                              byte[] var7;
                              int var31;
                              TlsUtils.readFully(var7 = new byte[var31 = TlsUtils.readUint16(var16)], var16);
                              int var8;
                              byte[] var9;
                              TlsUtils.readFully(var9 = new byte[var8 = TlsUtils.readUint16(var16)], var16);
                              this.assertEmpty(var16);
                              CombinedHash var33;
                              CombinedHash var41 = var33 = new CombinedHash;
                              var41.<init>();
                              ByteArrayOutputStream var10;
                              ByteArrayOutputStream var51 = var10 = new ByteArrayOutputStream;
                              var10.<init>();
                              TlsUtils.writeUint16(var15, var10);
                              var10.write(var23);
                              TlsUtils.writeUint16(var4, var10);
                              var10.write(var26);
                              TlsUtils.writeUint16(var31, var10);
                              var51.write(var7);
                              byte[] var53 = var51.toByteArray();
                              var10008 = this.clientRandom;
                              var33.update(var10008, 0, var10008.length);
                              var10006 = this.serverRandom;
                              var33.update(var10006, 0, var10006.length);
                              var41.update(var53, 0, var53.length);
                              var41.doFinal(var25 = new byte[var15 = var41.getDigestSize()], 0);
                              RSABlindedEngine var34;
                              var34 = new RSABlindedEngine.<init>();
                              PKCS1Encoding var32;
                              var40 = var32 = new PKCS1Encoding;
                              var44 = var9;
                              var32.<init>(var34);
                              var32.init(false, this.serverRsaKey);
                              var16 = null;
                              byte var47 = 0;

                              label132: {
                                 try {
                                    var39 = var40.processBlock(var44, var47, var8);
                                 } catch (InvalidCipherTextException var12) {
                                    this.failWithError((short)2, (short)42);
                                    var6 = (byte[])var16;
                                    break label132;
                                 }

                                 var2 = var39;
                                 var6 = var2;
                              }

                              if (var6.length != var15) {
                                 this.failWithError((short)2, (short)42);
                              }

                              for(var15 = 0; var15 < var6.length; ++var15) {
                                 if (var6[var15] != var25[var15]) {
                                    this.failWithError((short)2, (short)42);
                                 }
                              }

                              var36 = new BigInteger.<init>(1, var23);
                              BigInteger var35;
                              var35 = new BigInteger.<init>(1, var26);
                              BigInteger var50 = var27 = new BigInteger;
                              BigInteger var55 = var35;
                              var27.<init>(1, var7);
                              BigInteger var10005 = var35 = new BigInteger;
                              var3 = var36.bitLength() - 1;
                              var10005.<init>(var3, this.random);
                              this.Yc = var55.modPow(var10005, var36);
                              this.pms = BigIntegers.asUnsignedByteArray(var50.modPow(var35, var36));
                              this.connection_state = 4;
                              var1 = true;
                           }
                           break;
                        case 13:
                           short var24;
                           if ((var24 = this.connection_state) != 3) {
                              if (var24 != 4) {
                                 this.failWithError((short)2, (short)10);
                                 break;
                              }
                           } else if (this.choosenCipherSuite.getKeyExchangeAlgorithm() != 1) {
                              this.failWithError((short)2, (short)10);
                           }

                           TlsUtils.readFully(new byte[TlsUtils.readUint8(var16)], var16);
                           TlsUtils.readFully(new byte[TlsUtils.readUint16(var16)], var16);
                           this.assertEmpty(var16);
                           this.connection_state = 5;
                           var1 = true;
                           break;
                        case 14:
                           if ((var3 = this.connection_state) != 3) {
                              if (var3 != 4 && var3 != 5) {
                                 this.failWithError((short)2, (short)40);
                                 break;
                              }
                           } else if (this.choosenCipherSuite.getKeyExchangeAlgorithm() != 1) {
                              this.failWithError((short)2, (short)10);
                           }

                           this.assertEmpty(var16);
                           boolean var10000 = this.connection_state == 5;
                           this.connection_state = 6;
                           if (var10000) {
                              this.sendClientCertificate();
                           }

                           int var21;
                           ByteArrayOutputStream var22;
                           ByteArrayOutputStream var49;
                           if ((var15 = this.choosenCipherSuite.getKeyExchangeAlgorithm()) != 1) {
                              if (var15 != 5) {
                                 this.failWithError((short)2, (short)10);
                              } else {
                                 var20 = this.Yc.toByteArray();
                                 var49 = var22 = new ByteArrayOutputStream;
                                 var22.<init>();
                                 TlsUtils.writeUint8((short)16, var22);
                                 TlsUtils.writeUint24(var20.length + 2, var22);
                                 TlsUtils.writeUint16(var20.length, var22);
                                 var49.write(var20);
                                 var20 = var49.toByteArray();
                                 var21 = var20.length;
                                 this.rs.writeMessage((short)22, var20, 0, var21);
                              }
                           } else {
                              var39 = this.pms = new byte[48];
                              var39[0] = 3;
                              var39[1] = 1;

                              for(var15 = 2; var15 < 48; ++var15) {
                                 this.pms[var15] = (byte)this.random.nextInt();
                              }

                              RSABlindedEngine var17;
                              var17 = new RSABlindedEngine.<init>();
                              TlsProtocolHandler var42 = this;
                              var40 = new PKCS1Encoding(var17);
                              ParametersWithRandom var18;
                              ParametersWithRandom var10003 = var18 = new ParametersWithRandom;
                              var19 = this.serverRsaKey;
                              var10003.<init>(var19, this.random);
                              var40.init(true, var18);
                              var20 = null;

                              label154: {
                                 label153: {
                                    label181: {
                                       boolean var43;
                                       try {
                                          var44 = var42.pms;
                                       } catch (InvalidCipherTextException var14) {
                                          var43 = false;
                                          break label181;
                                       }

                                       byte[] var10002 = var44;
                                       var21 = 0;

                                       try {
                                          var39 = var40.processBlock(var44, var21, var10002.length);
                                          break label153;
                                       } catch (InvalidCipherTextException var13) {
                                          var43 = false;
                                       }
                                    }

                                    this.failWithError((short)2, (short)80);
                                    break label154;
                                 }

                                 var20 = var39;
                              }

                              var49 = var22 = new ByteArrayOutputStream;
                              var22.<init>();
                              TlsUtils.writeUint8((short)16, var22);
                              TlsUtils.writeUint24(var20.length + 2, var22);
                              TlsUtils.writeUint16(var20.length, var22);
                              var49.write(var20);
                              var20 = var49.toByteArray();
                              var21 = var20.length;
                              this.rs.writeMessage((short)22, var20, 0, var21);
                           }

                           this.connection_state = 7;
                           (var20 = new byte[1])[0] = 1;
                           this.rs.writeMessage((short)20, var20, 0, 1);
                           this.connection_state = 8;
                           this.ms = new byte[48];
                           var10008 = this.clientRandom;
                           var20 = new byte[var10008.length + this.serverRandom.length];
                           var21 = var10008.length;
                           System.arraycopy(var10008, 0, var20, 0, var21);
                           byte[] var10007 = this.serverRandom;
                           var21 = this.clientRandom.length;
                           var3 = var10007.length;
                           System.arraycopy(var10007, 0, var20, var21, var3);
                           var10006 = this.pms;
                           var2 = TlsUtils.toByteArray("master secret");
                           var23 = this.ms;
                           TlsUtils.PRF(var10006, var2, var20, var23);
                           RecordStream var28 = this.rs;
                           TlsCipherSuite var29;
                           TlsCipherSuite var10004 = var29 = this.choosenCipherSuite;
                           var28.writeSuite = var29;
                           var20 = this.ms;
                           var2 = this.clientRandom;
                           var23 = this.serverRandom;
                           var10004.init(var20, var2, var23);
                           var20 = new byte[12];
                           var2 = new byte[36];
                           this.rs.hash1.doFinal(var2, 0);
                           TlsUtils.PRF(this.ms, TlsUtils.toByteArray("client finished"), var2, var20);
                           ByteArrayOutputStream var46 = var22 = new ByteArrayOutputStream;
                           var22.<init>();
                           TlsUtils.writeUint8((short)20, var22);
                           TlsUtils.writeUint24(12, var22);
                           var46.write(var20);
                           var20 = var46.toByteArray();
                           var21 = var20.length;
                           this.rs.writeMessage((short)22, var20, 0, var21);
                           this.connection_state = 9;
                           var1 = true;
                           break;
                        default:
                           this.failWithError((short)2, (short)10);
                     }
                  } else if (this.connection_state != 10) {
                     this.failWithError((short)2, (short)10);
                  } else {
                     byte var30 = 12;
                     TlsUtils.readFully(var25 = new byte[12], var16);
                     this.assertEmpty(var16);
                     var26 = new byte[12];
                     var20 = new byte[36];
                     this.rs.hash2.doFinal(var20, 0);
                     TlsUtils.PRF(this.ms, TlsUtils.toByteArray("server finished"), var20, var26);

                     for(var15 = 0; var15 < var30; ++var15) {
                        if (var25[var15] != var26[var15]) {
                           this.failWithError((short)2, (short)40);
                        }
                     }

                     this.connection_state = 11;
                     this.appDataReady = true;
                     var1 = true;
                  }
               } else if (this.connection_state != 1) {
                  this.failWithError((short)2, (short)10);
               } else {
                  TlsUtils.checkVersion((InputStream)var16, this);
                  byte[] var56 = var20 = new byte[32];
                  this.serverRandom = var20;
                  TlsUtils.readFully(var56, var16);
                  TlsUtils.readFully(new byte[TlsUtils.readUint8(var16)], var16);
                  this.choosenCipherSuite = TlsCipherSuiteManager.getCipherSuite(TlsUtils.readUint16(var16), this);
                  if (TlsUtils.readUint8(var16) != 0) {
                     this.failWithError((short)2, (short)47);
                  }

                  this.assertEmpty(var16);
                  this.connection_state = 2;
                  var1 = true;
               }
            }
         }
      } while(var1);

   }

   private void processApplicationData() {
   }

   private void processAlert() {
      while(this.alertQueue.size() >= 2) {
         byte[] var1;
         byte[] var10000 = var1 = new byte[2];
         this.alertQueue.read(var1, 0, 2, 0);
         this.alertQueue.removeData(2);
         byte var4 = var10000[0];
         byte var3 = var1[1];
         if (var4 == 2) {
            TlsProtocolHandler var5 = this;
            this.failedWithError = true;
            this.closed = true;

            try {
               var5.rs.close();
            } catch (Exception var2) {
            }

            throw new IOException("Internal TLS error, this could be an attack");
         }

         if (var3 == 0) {
            this.failWithError((short)1, (short)0);
         }
      }

   }

   private void processChangeCipherSpec() {
      while(this.changeCipherSpecQueue.size() > 0) {
         byte[] var1;
         byte[] var10000 = var1 = new byte[1];
         this.changeCipherSpecQueue.read(var1, 0, 1, 0);
         this.changeCipherSpecQueue.removeData(1);
         if (var10000[0] != 1) {
            this.failWithError((short)2, (short)10);
         } else if (this.connection_state == 9) {
            RecordStream var10001 = this.rs;
            var10001.readSuite = var10001.writeSuite;
            this.connection_state = 10;
         } else {
            this.failWithError((short)2, (short)40);
         }
      }

   }

   private void sendClientCertificate() {
      TlsProtocolHandler var10000 = this;
      ByteArrayOutputStream var2;
      ByteArrayOutputStream var10001 = var2 = new ByteArrayOutputStream;
      var10001.<init>();
      TlsUtils.writeUint8((short)11, var2);
      TlsUtils.writeUint24(3, var2);
      TlsUtils.writeUint24(0, var2);
      byte[] var4 = var10001.toByteArray();
      RecordStream var3 = var10000.rs;
      int var1 = var4.length;
      var3.writeMessage((short)22, var4, 0, var1);
   }

   protected void processData(short protocol, byte[] buf, int offset, int len) {
      switch (protocol) {
         case 20:
            this.changeCipherSpecQueue.addData(buf, offset, len);
            this.processChangeCipherSpec();
            break;
         case 21:
            this.alertQueue.addData(buf, offset, len);
            this.processAlert();
            break;
         case 22:
            this.handshakeQueue.addData(buf, offset, len);
            this.processHandshake();
            break;
         case 23:
            if (!this.appDataReady) {
               this.failWithError((short)2, (short)10);
            }

            this.applicationDataQueue.addData(buf, offset, len);
            this.processApplicationData();
      }

   }

   public void connect(CertificateVerifyer verifyer) {
      this.verifyer = verifyer;
      this.clientRandom = new byte[32];
      int verifyer1 = (int)(System.currentTimeMillis() / 1000L);
      byte[] var2;
      byte[] var10000 = var2 = this.clientRandom;
      var2[0] = (byte)(verifyer1 >> 24);
      var2[1] = (byte)(verifyer1 >> 16);
      var2[2] = (byte)(verifyer1 >> 8);
      var10000[3] = (byte)verifyer1;

      for(verifyer1 = 4; verifyer1 < (var2 = this.clientRandom).length; ++verifyer1) {
         var2[verifyer1] = (byte)this.random.nextInt();
      }

      ByteArrayOutputStream verifyer2;
      ByteArrayOutputStream var10002 = verifyer2 = new ByteArrayOutputStream;
      verifyer2.<init>();
      TlsUtils.writeVersion(verifyer2);
      var10002.write(this.clientRandom);
      TlsUtils.writeUint8((short)0, verifyer2);
      TlsCipherSuiteManager.writeCipherSuites(var10002);
      byte[] var10003 = new byte[1];
      TlsUtils.writeUint8((short)1, verifyer2);
      var10002.write(var10003);
      ByteArrayOutputStream var5;
      var10002 = var5 = new ByteArrayOutputStream;
      var5.<init>();
      TlsUtils.writeUint8((short)1, var5);
      TlsUtils.writeUint24(verifyer2.size(), var5);
      var10002.write(verifyer2.toByteArray());
      byte[] verifyer3 = var10002.toByteArray();
      int var7 = verifyer3.length;
      this.rs.writeMessage((short)22, verifyer3, 0, var7);
      this.connection_state = 1;

      while(this.connection_state != 11) {
         this.rs.readData();
      }

      TlsInputStream verifyer4;
      verifyer4 = new TlsInputStream.<init>(this);
      this.tlsInputStream = verifyer4;
      TlsOuputStream verifyer5;
      verifyer5 = new TlsOuputStream.<init>(this);
      this.tlsOutputStream = verifyer5;
   }

   protected int readApplicationData(byte[] buf, int offset, int len) {
      while(this.applicationDataQueue.size() == 0) {
         if (this.failedWithError) {
            throw new IOException("Internal TLS error, this could be an attack");
         }

         if (this.closed) {
            return -1;
         }

         try {
            this.rs.readData();
         } catch (IOException var4) {
            if (!this.closed) {
               this.failWithError((short)2, (short)80);
            }

            throw var4;
         } catch (RuntimeException var5) {
            if (!this.closed) {
               this.failWithError((short)2, (short)80);
            }

            throw var5;
         }
      }

      int var10000 = len = Math.min(len, this.applicationDataQueue.size());
      this.applicationDataQueue.read(buf, offset, len, 0);
      this.applicationDataQueue.removeData(len);
      return var10000;
   }

   protected void writeData(byte[] buf, int offset, int len) {
      if (this.failedWithError) {
         throw new IOException("Internal TLS error, this could be an attack");
      } else if (this.closed) {
         throw new IOException("Sorry, connection has been closed, you cannot write more data");
      } else {
         this.rs.writeMessage((short)23, emptybuf, 0, 0);

         int var10000;
         do {
            var10000 = len;

            try {
               this.rs.writeMessage((short)23, buf, offset, len);
            } catch (IOException var4) {
               if (!this.closed) {
                  this.failWithError((short)2, (short)80);
               }

               throw var4;
            } catch (RuntimeException var5) {
               if (!this.closed) {
                  this.failWithError((short)2, (short)80);
               }

               throw var5;
            }

            offset += len = Math.min(len, 16384);
         } while((len = var10000 - len) > 0);

      }
   }

   /** @deprecated */
   public TlsOuputStream getTlsOuputStream() {
      return this.tlsOutputStream;
   }

   public OutputStream getOutputStream() {
      return this.tlsOutputStream;
   }

   /** @deprecated */
   public TlsInputStream getTlsInputStream() {
      return this.tlsInputStream;
   }

   public InputStream getInputStream() {
      return this.tlsInputStream;
   }

   protected void failWithError(short alertLevel, short alertDescription) {
      if (!this.closed) {
         byte[] var3;
         byte[] var10002 = var3 = new byte[2];
         var3[0] = (byte)alertLevel;
         var10002[1] = (byte)alertDescription;
         this.closed = true;
         if (alertLevel == 2) {
            this.failedWithError = true;
         }

         this.rs.writeMessage((short)21, var3, 0, 2);
         this.rs.close();
         if (alertLevel == 2) {
            throw new IOException("Internal TLS error, this could be an attack");
         }
      } else {
         throw new IOException("Internal TLS error, this could be an attack");
      }
   }

   public void close() {
      if (!this.closed) {
         this.failWithError((short)1, (short)0);
      }

   }

   protected void assertEmpty(ByteArrayInputStream is) {
      if (is.available() > 0) {
         this.failWithError((short)2, (short)50);
      }

   }

   protected void flush() {
      this.rs.flush();
   }
}
